`include "usb_audio_class/temp/USBAudioClass/uac_define.v"
`define LOOPBACK


module Top(
     input      CLK_12M
    ,input      CLK_IIS
    ,inout      USB_DXP_IO     
    ,inout      USB_DXN_IO     
    ,input      USB_RXDP_I     
    ,input      USB_RXDN_I     
    ,output     USB_PULLUP_EN_O
    ,inout      USB_TERM_DP_IO 
    ,inout      USB_TERM_DN_IO 
//speaker
    ,output     SPK_IIS_LRCK_O1     
    ,output     SPK_IIS_BCLK_O1     
    ,output     SPK_IIS_DATA_O1     

    ,output     SPK_IIS_LRCK_O2    
    ,output     SPK_IIS_BCLK_O2    
    ,output     SPK_IIS_DATA_O2    

    ,output     SPK_IIS_LRCK_O3    
    ,output     SPK_IIS_BCLK_O3    
    ,output     SPK_IIS_DATA_O3    
//mic

);

wire [1:0]  phy_xcvrselect;
wire        phy_termselect;
wire [1:0]  phy_opmode;
wire [1:0]  phy_linestate;
wire        phy_txvalid;
wire        phy_txready;
wire        phy_rxvalid;
wire        phy_rxactive;
wire        phy_rxerror;
wire [7:0]  phy_datain;
wire [7:0]  phy_dataout;
wire        phy_clkout;
wire        phy_reset;


wire [15:0] descrom_raddr;
wire [7:0]  desc_index;
wire [7:0]  desc_type;
wire [7:0]  descrom_rdat;
wire [15:0] desc_dev_addr;
wire [15:0] desc_dev_len;
wire [15:0] desc_qual_addr;
wire [15:0] desc_qual_len;
wire [15:0] desc_fscfg_addr;
wire [15:0] desc_fscfg_len;
wire [15:0] desc_hscfg_addr;
wire [15:0] desc_hscfg_len;
wire [15:0] desc_oscfg_addr;
wire [15:0] desc_hidrpt_addr;
wire [15:0] desc_hidrpt_len;
wire [15:0] desc_strlang_addr;
wire [15:0] desc_strvendor_addr;
wire [15:0] desc_strvendor_len;
wire [15:0] desc_strproduct_addr;
wire [15:0] desc_strproduct_len;
wire [15:0] desc_strserial_addr;
wire [15:0] desc_strserial_len;
wire        descrom_have_strings;


wire        usb_busreset;
wire        usb_highspeed;
wire        usb_suspend;
wire        usb_online;
wire        usb_txval;
wire [ 7:0] usb_txdat;
wire [11:0] usb_txdat_len;
wire        usb_txcork;
wire        usb_txpop;
wire        usb_txact;
wire        usb_txpktfin;
wire [ 7:0] usb_rxdat;
wire        usb_rxval;
wire        usb_rxact;
wire        usb_rxpktval;
wire        usb_setup;
wire [ 3:0] usb_endpt_sel;
wire        usb_sof;
wire [ 7:0] interface_alter_i;
wire [ 7:0] interface_alter_o;
wire [ 7:0] interface_sel;
wire        interface_update;


//==============================================================
//======PLL 
wire        fclk_480M;
wire        pll_locked_1;
wire        pll_locked_2;
wire        CLK_98304;

    Gowin_rPLL Gowin_rPLL(
        .clkout     (fclk_480M      ), //output clkout
        .lock       (pll_locked_1   ), //output lock
        .clkoutd    (phy_clkout     ), //output clkoutd
        .clkin      (CLK_12M        ) //input clkin
    );

    iis_rPLL iis_rPLL(
        .clkout     (CLK_98304      ), //output clkout
        .lock       (pll_locked_2   ), //output lock
        .clkin      (CLK_IIS        ) //input clkin
    );

assign reset = ~(pll_locked_1 & pll_locked_2);

//==============================================================
//======Device Controller
`ifdef HS 
USB_Device_Controller_Top_HS u_usb_device_controller_top_HS (
         .clk_i                 (phy_clkout          )
        ,.reset_i               (reset               )
        ,.usbrst_o              (usb_busreset        )
        ,.highspeed_o           (usb_highspeed       )
        ,.suspend_o             (usb_suspend         )
        ,.online_o              (usb_online          )
        ,.txdat_i               (usb_txdat           )
        ,.txval_i               (usb_txval           )
        ,.txdat_len_i           (usb_txdat_len       )
        ,.txcork_i              (usb_txcork          )
        ,.txiso_pid_i           (4'b0011             )
        ,.txpop_o               (usb_txpop           )
        ,.txact_o               (usb_txact           )
        ,.txpktfin_o            (usb_txpktfin        )
        ,.rxdat_o               (usb_rxdat           )
        ,.rxval_o               (usb_rxval           )
        ,.rxact_o               (usb_rxact           )
        ,.rxrdy_i               (1'b1                )
        ,.rxpktval_o            (usb_rxpktval        )
        ,.setup_o               (usb_setup           )
        ,.endpt_o               (usb_endpt_sel       )
        ,.sof_o                 (usb_sof             )
        ,.inf_alter_i           (interface_alter_i   )
        ,.inf_alter_o           (interface_alter_o   )
        ,.inf_sel_o             (interface_sel       )
        ,.inf_set_o             (interface_update    )
        ,.descrom_rdata_i       (descrom_rdat        )
        ,.descrom_raddr_o       (descrom_raddr       )
        ,.desc_index_o          (desc_index          )
        ,.desc_type_o           (desc_type           )
        ,.desc_dev_addr_i       (desc_dev_addr       )
        ,.desc_dev_len_i        (desc_dev_len        )
        ,.desc_qual_addr_i      (desc_qual_addr      )
        ,.desc_qual_len_i       (desc_qual_len       )
        ,.desc_fscfg_addr_i     (desc_fscfg_addr     )
        ,.desc_fscfg_len_i      (desc_fscfg_len      )
        ,.desc_hscfg_addr_i     (desc_hscfg_addr     )
        ,.desc_hscfg_len_i      (desc_hscfg_len      )
        ,.desc_oscfg_addr_i     (desc_oscfg_addr     )
        ,.desc_hidrpt_addr_i    (desc_hidrpt_addr    )
        ,.desc_hidrpt_len_i     (desc_hidrpt_len     )
        ,.desc_strlang_addr_i   (desc_strlang_addr   )
        ,.desc_strvendor_addr_i (desc_strvendor_addr )
        ,.desc_strvendor_len_i  (desc_strvendor_len  )
        ,.desc_strproduct_addr_i(desc_strproduct_addr)
        ,.desc_strproduct_len_i (desc_strproduct_len )
        ,.desc_strserial_addr_i (desc_strserial_addr )
        ,.desc_strserial_len_i  (desc_strserial_len  )
        ,.desc_bos_addr_i       (16'd0                   )
        ,.desc_bos_len_i        (16'd0                   )
        ,.desc_have_strings_i   (descrom_have_strings)
        ,.utmi_dataout_o        (phy_dataout         )
        ,.utmi_txvalid_o        (phy_txvalid         )
        ,.utmi_txready_i        (phy_txready         )
        ,.utmi_datain_i         (phy_datain          )
        ,.utmi_rxactive_i       (phy_rxactive        )
        ,.utmi_rxvalid_i        (phy_rxvalid         )
        ,.utmi_rxerror_i        (phy_rxerror         )
        ,.utmi_linestate_i      (phy_linestate       )
        ,.utmi_opmode_o         (phy_opmode          )
        ,.utmi_xcvrselect_o     (phy_xcvrselect      )
        ,.utmi_termselect_o     (phy_termselect      )
        ,.utmi_reset_o          (phy_reset           )
);
`endif

`ifdef FS
USB_Device_Controller_Top_FS u_usb_device_controller_top_FS (
         .clk_i                 (phy_clkout          )
        ,.reset_i               (reset               )
        ,.usbrst_o              (usb_busreset        )
        ,.highspeed_o           (usb_highspeed       )
        ,.suspend_o             (usb_suspend         )
        ,.online_o              (usb_online          )
        ,.txdat_i               (usb_txdat           )
        ,.txval_i               (usb_txval           )
        ,.txdat_len_i           (usb_txdat_len       )
        ,.txcork_i              (usb_txcork          )
        ,.txiso_pid_i           (4'b0011             )
        ,.txpop_o               (usb_txpop           )
        ,.txact_o               (usb_txact           )
        ,.txpktfin_o            (usb_txpktfin        )
        ,.rxdat_o               (usb_rxdat           )
        ,.rxval_o               (usb_rxval           )
        ,.rxact_o               (usb_rxact           )
        ,.rxrdy_i               (1'b1                )
        ,.rxpktval_o            (usb_rxpktval        )
        ,.setup_o               (usb_setup           )
        ,.endpt_o               (usb_endpt_sel       )
        ,.sof_o                 (usb_sof             )
        ,.inf_alter_i           (interface_alter_i   )
        ,.inf_alter_o           (interface_alter_o   )
        ,.inf_sel_o             (interface_sel       )
        ,.inf_set_o             (interface_update    )
        ,.descrom_rdata_i       (descrom_rdat        )
        ,.descrom_raddr_o       (descrom_raddr       )
        ,.desc_index_o          (desc_index          )
        ,.desc_type_o           (desc_type           )
        ,.desc_dev_addr_i       (desc_dev_addr       )
        ,.desc_dev_len_i        (desc_dev_len        )
        ,.desc_qual_addr_i      (desc_qual_addr      )
        ,.desc_qual_len_i       (desc_qual_len       )
        ,.desc_fscfg_addr_i     (desc_fscfg_addr     )
        ,.desc_fscfg_len_i      (desc_fscfg_len      )
        ,.desc_hscfg_addr_i     (desc_hscfg_addr     )
        ,.desc_hscfg_len_i      (desc_hscfg_len      )
        ,.desc_oscfg_addr_i     (desc_oscfg_addr     )
        ,.desc_hidrpt_addr_i    (desc_hidrpt_addr    )
        ,.desc_hidrpt_len_i     (desc_hidrpt_len     )
        ,.desc_strlang_addr_i   (desc_strlang_addr   )
        ,.desc_strvendor_addr_i (desc_strvendor_addr )
        ,.desc_strvendor_len_i  (desc_strvendor_len  )
        ,.desc_strproduct_addr_i(desc_strproduct_addr)
        ,.desc_strproduct_len_i (desc_strproduct_len )
        ,.desc_strserial_addr_i (desc_strserial_addr )
        ,.desc_strserial_len_i  (desc_strserial_len  )
        ,.desc_bos_addr_i       (16'd0                   )
        ,.desc_bos_len_i        (16'd0                   )
        ,.desc_have_strings_i   (descrom_have_strings)
        ,.utmi_dataout_o        (phy_dataout         )
        ,.utmi_txvalid_o        (phy_txvalid         )
        ,.utmi_txready_i        (phy_txready         )
        ,.utmi_datain_i         (phy_datain          )
        ,.utmi_rxactive_i       (phy_rxactive        )
        ,.utmi_rxvalid_i        (phy_rxvalid         )
        ,.utmi_rxerror_i        (phy_rxerror         )
        ,.utmi_linestate_i      (phy_linestate       )
        ,.utmi_opmode_o         (phy_opmode          )
        ,.utmi_xcvrselect_o     (phy_xcvrselect      )
        ,.utmi_termselect_o     (phy_termselect      )
        ,.utmi_reset_o          (phy_reset           )
);
`endif
//==============================================================
//======USB SoftPHY
USB2_0_SoftPHY_Top u_USB_SoftPHY_Top
(
     .clk_i             (phy_clkout     )
    ,.rst_i             (phy_reset      )
    ,.fclk_i            (fclk_480M      )
    ,.pll_locked_i      (pll_locked_1   )
    ,.utmi_data_out_i   (phy_dataout    )
    ,.utmi_txvalid_i    (phy_txvalid    )
    ,.utmi_op_mode_i    (phy_opmode     )
    ,.utmi_xcvrselect_i (phy_xcvrselect )
    ,.utmi_termselect_i (phy_termselect )
    ,.utmi_data_in_o    (phy_datain     )
    ,.utmi_txready_o    (phy_txready    )
    ,.utmi_rxvalid_o    (phy_rxvalid    )
    ,.utmi_rxactive_o   (phy_rxactive   )
    ,.utmi_rxerror_o    (phy_rxerror    )
    ,.utmi_linestate_o  (phy_linestate  )
    ,.usb_dxp_io        (USB_DXP_IO     )
    ,.usb_dxn_io        (USB_DXN_IO     )
    ,.usb_rxdp_i        (USB_RXDP_I     )
    ,.usb_rxdn_i        (USB_RXDN_I     )
    ,.usb_pullup_en_o   (USB_PULLUP_EN_O)
    ,.usb_term_dp_io    (USB_TERM_DP_IO )
    ,.usb_term_dn_io    (USB_TERM_DN_IO )
);

//==============================================================
//======UAC
USB_Audio_Class_Top 
/*
#(
     .VENDORID               (16'h33AA)
    ,.PRODUCTID              (16'h0721)
    ,.VERSIONBCD             (16'h0116)
)
*/
USB_Audio_Class_3Speaker3Mic(
     .i_clk60               (phy_clkout             )
    ,.i_clk98304            (CLK_98304              )

    ,.i_reset               (reset                  )
    ,.i_usb_busreset        (usb_busreset           )
//====================descriptor==============================
    ,.i_desc_raddr          (descrom_raddr          )
    ,.o_desc_rdat           (descrom_rdat           )
    ,.i_desc_index          (desc_index             )
    ,.i_desc_type           (desc_type              )
    ,.o_desc_dev_addr       (desc_dev_addr          )
    ,.o_desc_dev_len        (desc_dev_len           )
    ,.o_desc_qual_addr      (desc_qual_addr         )
    ,.o_desc_qual_len       (desc_qual_len          )
    ,.o_desc_fscfg_addr     (desc_fscfg_addr        )
    ,.o_desc_fscfg_len      (desc_fscfg_len         )
    ,.o_desc_hscfg_addr     (desc_hscfg_addr        )
    ,.o_desc_hscfg_len      (desc_hscfg_len         )
    ,.o_desc_hidrpt_addr    (desc_oscfg_addr        )
    ,.o_desc_hidrpt_len     (desc_hidrpt_addr       )
    ,.o_desc_oscfg_addr     (desc_hidrpt_len        )
    ,.o_desc_strlang_addr   (desc_strlang_addr      )
    ,.o_desc_strvendor_addr (desc_strvendor_addr    )
    ,.o_desc_strvendor_len  (desc_strvendor_len     )
    ,.o_desc_strproduct_addr(desc_strproduct_addr   )
    ,.o_desc_strproduct_len (desc_strproduct_len    )
    ,.o_desc_strserial_addr (desc_strserial_addr    )
    ,.o_desc_strserial_len  (desc_strserial_len     )
    ,.o_desc_have_strings   (descrom_have_strings   )
//========================USB==============================
    ,.i_usb_setup           (usb_setup              )
    ,.i_usb_endpt_sel       (usb_endpt_sel          )

    ,.i_usb_sof             (usb_sof                )

    ,.i_usb_txact           (usb_txact              )
    ,.i_usb_txpop           (usb_txpop              )
    ,.o_usb_txval           (usb_txval              )
    ,.o_usb_txdat           (usb_txdat              )
    ,.o_usb_txdat_len       (usb_txdat_len          )
    ,.i_usb_txpktfin        (usb_txpktfin           )
    ,.o_usb_txcork          (usb_txcork             )

    ,.i_usb_rxact           (usb_rxact              )
    ,.i_usb_rxval           (usb_rxval              )
    ,.i_usb_rxpktval        (usb_rxpktval           )
    ,.o_usb_rxrdy           (                       )//no care
    ,.i_usb_rxdat           (usb_rxdat              )

    ,.i_interface_alter     (interface_alter_o      )
    ,.o_interface_alter     (interface_alter_i      )
    ,.i_interface_sel       (interface_sel          )
    ,.i_interface_update    (interface_update       )

//====================audio_tx==============================
`ifdef ENABLE_SPEAKER1
    `ifdef SPEAKER1_IIS
    ,.o_spk_iis_bclk_1      (o_spk_iis_bclk_1)
    ,.o_spk_iis_lrck_1      (o_spk_iis_lrck_1)
    ,.o_spk_iis_data_1      (o_spk_iis_data_1)
    `endif
    `ifdef SPEAKER1_PCM
    ,.o_spk_pcm_bclk_1      (o_spk_pcm_bclk_1) 
    ,.o_spk_pcm_lrck_1      (o_spk_pcm_lrck_1)  
    ,.o_spk_pcm_data_1      (o_spk_pcm_data_1)  
    `endif
    `ifdef SPEAKER1_PCM_32
    ,.o_spk_l_pcm_data_1    (o_spk_l_pcm_data_1)
    ,.o_spk_r_pcm_data_1    (o_spk_r_pcm_data_1)
    `endif
`endif

`ifdef ENABLE_SPEAKER2
    `ifdef SPEAKER2_IIS
    ,.o_spk_iis_bclk_2      (o_spk_iis_bclk_2)
    ,.o_spk_iis_lrck_2      (o_spk_iis_lrck_2)
    ,.o_spk_iis_data_2      (o_spk_iis_data_2)
    `endif
    `ifdef SPEAKER2_PCM
    ,.o_spk_pcm_bclk_2      (o_spk_pcm_bclk_2) 
    ,.o_spk_pcm_lrck_2      (o_spk_pcm_lrck_2)  
    ,.o_spk_pcm_data_2      (o_spk_pcm_data_2)  
    `endif
    `ifdef SPEAKER2_PCM_32
    ,.o_spk_l_pcm_data_2    (o_spk_l_pcm_data_2)
    ,.o_spk_r_pcm_data_2    (o_spk_r_pcm_data_2)
    `endif
`endif

`ifdef ENABLE_SPEAKER3
    `ifdef SPEAKER3_IIS
    ,.o_spk_iis_bclk_3      (o_spk_iis_bclk_3)
    ,.o_spk_iis_lrck_3      (o_spk_iis_lrck_3)
    ,.o_spk_iis_data_3      (o_spk_iis_data_3)
    `endif
    `ifdef SPEAKER3_PCM
    ,.o_spk_pcm_bclk_3      (o_spk_pcm_bclk_3) 
    ,.o_spk_pcm_lrck_3      (o_spk_pcm_lrck_3)  
    ,.o_spk_pcm_data_3      (o_spk_pcm_data_3)  
    `endif
    `ifdef SPEAKER3_PCM_32
    ,.o_spk_l_pcm_data_3    (o_spk_l_pcm_data_3)
    ,.o_spk_r_pcm_data_3    (o_spk_r_pcm_data_3)
    `endif
`endif
//====================audio_rx==============================
`ifdef OUTSIDE_CLK
    `ifdef LOOPBACK
        `ifdef ENABLE_MIC1
    ,.i_mic_iis_bclk_1        (o_spk_iis_bclk_1  )
    ,.i_mic_iis_lrck_1        (o_spk_iis_lrck_1  )
    ,.i_mic_iis_data_1        (o_spk_iis_data_1  )
        `endif
        `ifdef ENABLE_MIC2
    ,.i_mic_iis_bclk_2        (o_spk_iis_bclk_2 )
    ,.i_mic_iis_lrck_2        (o_spk_iis_lrck_2 )
    ,.i_mic_iis_data_2        (o_spk_iis_data_2 )
        `endif
        `ifdef ENABLE_MIC3
    ,.i_mic_iis_bclk_3        (o_spk_iis_bclk_3 )   
    ,.i_mic_iis_lrck_3        (o_spk_iis_lrck_3 )
    ,.i_mic_iis_data_3        (o_spk_iis_data_3 )
        `endif
    `else
        `ifdef ENABLE_MIC1
    ,.i_mic_iis_bclk_1        (i_mic_iis_bclk_1  )
    ,.i_mic_iis_lrck_1        (i_mic_iis_lrck_1  )
    ,.i_mic_iis_data_1        (i_mic_iis_data_1  )
        `endif
        `ifdef ENABLE_MIC2
    ,.i_mic_iis_bclk_2        (i_mic_iis_bclk_2 )
    ,.i_mic_iis_lrck_2        (i_mic_iis_lrck_2 )
    ,.i_mic_iis_data_2        (i_mic_iis_data_2 )
        `endif
        `ifdef ENABLE_MIC3
    ,.i_mic_iis_bclk_3        (i_mic_iis_bclk_3 )   
    ,.i_mic_iis_lrck_3        (i_mic_iis_lrck_3 )
    ,.i_mic_iis_data_3        (i_mic_iis_data_3 )
        `endif
    `endif
`else
`ifdef ENABLE_MIC1
    ,.o_mic_iis_bclk_1        (o_mic_iis_bclk_1   )
    ,.o_mic_iis_lrck_1        (o_mic_iis_lrck_1   )
    ,.i_mic_iis_data_1        (i_mic_iis_data_1   )
`endif
`ifdef ENABLE_MIC2
    ,.o_mic_iis_bclk_2        (o_mic_iis_bclk_2   )
    ,.o_mic_iis_lrck_2        (o_mic_iis_lrck_2   )
    ,.i_mic_iis_data_2        (i_mic_iis_data_2   )
`endif
`ifdef ENABLE_MIC3
    ,.o_mic_iis_bclk_3        (o_mic_iis_bclk_3   )
    ,.o_mic_iis_lrck_3        (o_mic_iis_lrck_3   )
    ,.i_mic_iis_data_3        (i_mic_iis_data_3   )
`endif

`endif
);



// IO PORT

`ifdef ENABLE_SPEAKER1
assign SPK_IIS_LRCK_O1 =  o_spk_iis_lrck_1;
assign SPK_IIS_BCLK_O1 =  o_spk_iis_bclk_1;
assign SPK_IIS_DATA_O1 =  o_spk_iis_data_1;
`endif

`ifdef ENABLE_SPEAKER2
assign SPK_IIS_LRCK_O2 =  o_spk_iis_lrck_2;
assign SPK_IIS_BCLK_O2 =  o_spk_iis_bclk_2;
assign SPK_IIS_DATA_O2 =  o_spk_iis_data_2;
`endif

`ifdef ENABLE_SPEAKER3
assign SPK_IIS_LRCK_O3 =  o_spk_iis_lrck_3;
assign SPK_IIS_BCLK_O3 =  o_spk_iis_bclk_3;
assign SPK_IIS_DATA_O3 =  o_spk_iis_data_3;
`endif


endmodule